@@ -92,6 +92,8 @@ class Config_:
             # which override them, e.g. runner tests around warn=True, etc).
             expected = {
                 "run": {
+                    "asynchronous": False,
+                    "disown": False,
                     "dry": False,
                     "echo": False,
                     "echo_stdin": None,
@@ -641,7 +643,7 @@ Valid real attributes: ['clear', 'clone', 'env_prefix', 'file_prefix', 'from_dat
             c._load_yml = Mock(side_effect=IOError(2, "aw nuts"))
             c.set_runtime_path("is-a.yml")  # Triggers use of _load_yml
             c.load_runtime()
-            mock_debug.assert_has_call("Didn't see any is-a.yml, skipping.")
+            mock_debug.assert_any_call("Didn't see any is-a.yml, skipping.")
 
         @raises(IOError)
         def non_missing_file_IOErrors_are_raised(self):
