@@ -3,6 +3,7 @@ import signal
 import struct
 import sys
 import termios
+import threading
 import types
 
 from io import BytesIO
@@ -15,19 +16,20 @@ from pytest_relaxed import trap
 from mock import patch, Mock, call
 
 from invoke import (
-    Runner,
-    Local,
     CommandTimedOut,
-    Context,
     Config,
+    Context,
     Failure,
-    ThreadException,
-    SubprocessPipeError,
+    Local,
+    Promise,
     Responder,
-    WatcherError,
-    UnexpectedExit,
-    StreamWatcher,
     Result,
+    Runner,
+    StreamWatcher,
+    SubprocessPipeError,
+    ThreadException,
+    UnexpectedExit,
+    WatcherError,
 )
 from invoke.runners import default_encoding
 from invoke.terminals import WINDOWS
@@ -120,6 +122,8 @@ class _TimingOutRunner(_Dummy):
 
 
 class Runner_:
+    _stop_methods = ["generate_result", "stop", "stop_timer"]
+
     # NOTE: these copies of _run and _runner form the base case of "test Runner
     # subclasses via self._run/_runner helpers" functionality. See how e.g.
     # Local_ uses the same approach but bakes in the dummy class used.
@@ -482,6 +486,13 @@ class Runner_:
             assert out.getvalue() == "sup"
             assert sys.stdout.getvalue() == ""
 
+        @trap
+        def overridden_out_is_never_hidden(self):
+            out = StringIO()
+            self._runner(out="sup").run(_, out_stream=out, hide=True)
+            assert out.getvalue() == "sup"
+            assert sys.stdout.getvalue() == ""
+
         @trap
         def err_can_be_overridden(self):
             "err_stream can be overridden"
@@ -490,6 +501,13 @@ class Runner_:
             assert err.getvalue() == "sup"
             assert sys.stderr.getvalue() == ""
 
+        @trap
+        def overridden_err_is_never_hidden(self):
+            err = StringIO()
+            self._runner(err="sup").run(_, err_stream=err, hide=True)
+            assert err.getvalue() == "sup"
+            assert sys.stderr.getvalue() == ""
+
         @trap
         def pty_defaults_to_sys(self):
             self._runner(out="sup").run(_, pty=True)
@@ -1385,6 +1403,97 @@ Stderr: already printed
                 runner.run(_)
             runner.stop.assert_called_once_with()
 
+    class asynchronous:
+        def returns_Promise_immediately_and_finishes_on_join(self):
+            # Dummy subclass with controllable process_is_finished flag
+            class _Finisher(_Dummy):
+                _finished = False
+
+                @property
+                def process_is_finished(self):
+                    return self._finished
+
+            runner = _Finisher(Context())
+            # Set up mocks and go
+            runner.start = Mock()
+            for method in self._stop_methods:
+                setattr(runner, method, Mock())
+            result = runner.run(_, asynchronous=True)
+            # Got a Promise (its attrs etc are in its own test subsuite)
+            assert isinstance(result, Promise)
+            # Started, but did not stop (as would've happened for disown)
+            assert runner.start.called
+            for method in self._stop_methods:
+                assert not getattr(runner, method).called
+            # Set proc completion flag to truthy and join()
+            runner._finished = True
+            result.join()
+            for method in self._stop_methods:
+                assert getattr(runner, method).called
+
+        @trap
+        def hides_output(self):
+            # Run w/ faux subproc stdout/err data, but async
+            self._runner(out="foo", err="bar").run(_, asynchronous=True).join()
+            # Expect that default out/err streams did not get printed to.
+            assert sys.stdout.getvalue() == ""
+            assert sys.stderr.getvalue() == ""
+
+        def does_not_forward_stdin(self):
+            class MockedHandleStdin(_Dummy):
+                pass
+
+            MockedHandleStdin.handle_stdin = Mock()
+            runner = self._runner(klass=MockedHandleStdin)
+            runner.run(_, asynchronous=True).join()
+            # As with the main test for setting this to False, we know that
+            # when stdin is disabled, the handler is never even called (no
+            # thread is created for it).
+            assert not MockedHandleStdin.handle_stdin.called
+
+        def leaves_overridden_streams_alone(self):
+            # NOTE: technically a duplicate test of the generic tests for #637
+            # re: intersect of hide and overridden streams. But that's an
+            # implementation detail so this is still valuable.
+            klass = self._mock_stdin_writer()
+            out, err, in_ = StringIO(), StringIO(), StringIO("hallo")
+            runner = self._runner(out="foo", err="bar", klass=klass)
+            runner.run(
+                _,
+                asynchronous=True,
+                out_stream=out,
+                err_stream=err,
+                in_stream=in_,
+            ).join()
+            assert out.getvalue() == "foo"
+            assert err.getvalue() == "bar"
+            assert klass.write_proc_stdin.called  # lazy
+
+    class disown:
+        @patch.object(threading.Thread, "start")
+        def starts_and_returns_None_but_does_nothing_else(self, thread_start):
+            runner = Runner(Context())
+            runner.start = Mock()
+            not_called = self._stop_methods + ["wait"]
+            for method in not_called:
+                setattr(runner, method, Mock())
+            result = runner.run(_, disown=True)
+            # No Result object!
+            assert result is None
+            # Subprocess kicked off
+            assert runner.start.called
+            # No timer or IO threads started
+            assert not thread_start.called
+            # No wait or shutdown related Runner methods called
+            for method in not_called:
+                assert not getattr(runner, method).called
+
+        def cannot_be_given_alongside_asynchronous(self):
+            with raises(ValueError) as info:
+                self._runner().run(_, asynchronous=True, disown=True)
+            sentinel = "Cannot give both 'asynchronous' and 'disown'"
+            assert sentinel in str(info.value)
+
 
 class _FastLocal(Local):
     # Neuter this for same reason as in _Dummy above
@@ -1475,6 +1584,12 @@ class Local_:
                 assert e.type == OSError
                 assert str(e.value) == "wat"
 
+        @mock_pty(os_close_error=True)
+        def stop_mutes_errors_on_pty_close(self):
+            # Another doesn't-blow-up test, this time around os.close() of the
+            # pty itself (due to os_close_error=True)
+            self._run(_, pty=True)
+
         class fallback:
             @mock_pty(isatty=False)
             def can_be_overridden_by_kwarg(self):
@@ -1647,3 +1762,63 @@ class Result_:
         def encodes_with_result_encoding(self, encode):
             Result(stdout="foo", encoding="utf-16").tail("stdout")
             encode.assert_called_once_with("\n\nfoo", "utf-16")
+
+
+class Promise_:
+    def exposes_read_only_run_params(self):
+        runner = _runner()
+        promise = runner.run(
+            _, pty=True, encoding="utf-17", shell="sea", asynchronous=True
+        )
+        assert promise.command == _
+        assert promise.pty is True
+        assert promise.encoding == "utf-17"
+        assert promise.shell == "sea"
+        assert not hasattr(promise, "stdout")
+        assert not hasattr(promise, "stderr")
+
+    class join:
+        # NOTE: high level Runner lifecycle mechanics of join() (re: wait(),
+        # process_is_finished() etc) are tested in main suite.
+
+        def returns_Result_on_success(self):
+            result = _runner().run(_, asynchronous=True).join()
+            assert isinstance(result, Result)
+            # Sanity
+            assert result.command == _
+            assert result.exited == 0
+
+        def raises_main_thread_exception_on_kaboom(self):
+            runner = _runner(klass=_GenericExceptingRunner)
+            with raises(_GenericException):
+                runner.run(_, asynchronous=True).join()
+
+        def raises_subthread_exception_on_their_kaboom(self):
+            class Kaboom(_Dummy):
+                def handle_stdout(self, **kwargs):
+                    raise OhNoz()
+
+            runner = _runner(klass=Kaboom)
+            promise = runner.run(_, asynchronous=True)
+            with raises(ThreadException) as info:
+                promise.join()
+            assert isinstance(info.value.exceptions[0].value, OhNoz)
+
+        def raises_Failure_on_failure(self):
+            runner = _runner(exits=1)
+            promise = runner.run(_, asynchronous=True)
+            with raises(Failure):
+                promise.join()
+
+    class context_manager:
+        def calls_join_or_wait_on_close_of_block(self):
+            promise = _runner().run(_, asynchronous=True)
+            promise.join = Mock()
+            with promise:
+                pass
+            promise.join.assert_called_once_with()
+
+        def yields_self(self):
+            promise = _runner().run(_, asynchronous=True)
+            with promise as value:
+                assert value is promise
